// File:           bllib.h - constant & class definitions, public variables,
//                 function & method prototypes for using BLlib
//                 -*- C++ -*-
//
// Package:        Library
//
// Author:         Simone Zanella
//
// E-Mail:         szanella@dsi.unive.it
//
// Address:        via Gramsci, 72/22
//                 30035 Mirano (VE)
//                 ITALY
//
// Copyright (C) 1997 Simone Zanella Productions
//
// This program is free software; you can redistribute
// it and/or modify it under the terms of GNU General Public License
// as published by the Free Software Foundation; either version 2 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implicit warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// History:
//
// $Log:	bllib.h,v $
// Revision 1.13  97/03/28  12:00:00  12:00:00  szanella (Zanella Simone)
// first stand-alone public release
//


#ifndef _BL_MSG

 #define _BL_MSG

 //******************************************************************
 //************************ INCLUDE SECTION *************************
 //******************************************************************

 #include "bllocal.h"

 #include <stdlib.h>
 #include <iostream.h>
 #include <string.h>


 //******************************************************************
 //************************* TYPE SECTION ***************************
 //******************************************************************

 typedef long BL_INT32;

 typedef char * BL_Charp;     // z-string

 typedef struct               // binary object
 {
   char * pcData;
   unsigned long ulDataLen;
 } BL_RawData;

 // Integers of various extension, signed & unsigned, used by the
 // BL_ToIntxx / BL_ToUintxx functions

 typedef struct
 {
   char value[2];
 } BL_Int16;

 typedef struct
 {
   char value[2];
 } BL_Uint16;

 typedef struct
 {
   char value[4];
 } BL_Int32;

 typedef struct
 {
   char value[4];
 } BL_Uint32;

 typedef struct
 {
   char value[8];
 } BL_Int64;

 typedef struct
 {
   char value[8];
 } BL_Uint64;


 //******************************************************************
 //******************* PUBLIC VARIABLES SECTION *********************
 //******************************************************************

 extern int BL_convResult;
 extern BL_Int16 BL_i16;
 extern BL_Uint16 BL_ui16;
 extern BL_Int32 BL_i32;
 extern BL_Uint32 BL_ui32;
 extern BL_Int64 BL_i64;
 extern BL_Uint64 BL_ui64;
 extern BL_RawData BL_raw;


 //******************************************************************
 //*********************** CONSTANT SECTION *************************
 //******************************************************************

 // Constant definitions.

 // User message types should be over BL_RAWDATA or under BL_NOT_VALID

 // Preset types for message type tag
 enum BL_PresetTypes    {BL_NOT_VALID = -10,  BL_NOT_DEFINED,
                         BL_ALL_TYPE = 0,  BL_ALL_INT,   BL_ALL_FLOAT,
                         BL_CHAR,  BL_I16,    BL_I32,    BL_I64,
                         BL_UCHAR, BL_UI16,   BL_UI32,   BL_UI64,
                         BL_FLOAT, BL_DOUBLE, BL_CHARP,  BL_RAWDATA};


 // Results of conversion / extraction functions
 enum BL_ConvResults    {BL_NO_ITEM = -10, BL_MISMATCH,    BL_NEG_TO_POS,
                         BL_CANT_SHRINK,   BL_CONV_OK = 0, BL_FP_TO_INT,
                         BL_INT_TO_FP,     BL_PREC_LOSS};

 #define BL_ITEM_DELETED   0
 #define BL_WRONG_ITEM     -1


 //******************************************************************
 //******************** OTHER FUNCTIONS SECTION *********************
 //******************************************************************

 // Function to check result of allocation functions

 char *
 BL_ChkNul(char *);


 // Functions used to convert an integer to the internal memory format.

 BL_Int16
 BL_ToInt16(long);


 BL_Uint16
 BL_ToUint16(unsigned long);


 BL_Int32
 BL_ToInt32(long);


 BL_Uint32
 BL_ToUint32(unsigned long);


 BL_Int64
 BL_ToInt64(long);


 BL_Uint64
 BL_ToUint64(unsigned long);


 // Function to convert a binary object to a static structure suitable for
 // using in the composition of the message.

 BL_RawData
 BL_ToRawData(char *, unsigned long);


 //******************************************************************
 //************************* CLASS SECTION **************************
 //******************************************************************

 // BL_Message class: defines the attributes, constructor & destructor,
 // methods to receive & manipulate messages, methods to put base
 // values into the message, to set the type tag and to send the created
 // message to an addressee.

 class BL_Message

 {

   char * cpMsgBuf;              // Space to keep proper message data
   BL_INT32 i32Type;             // Message type
   unsigned long ulDataLen;      // Message length
   unsigned long ulBufOffset;    // Position of item cursor in the buffer
   unsigned long ulCurrentItem;  // Current item number

 public:

   // Constructors & destructor

   BL_Message();

   BL_Message(BL_Message&);       // Copy constructor

   ~BL_Message();

   // Extraction operators

   BL_Message& operator>>(char &);
   BL_Message& operator>>(unsigned char &);
   BL_Message& operator>>(short &);
   BL_Message& operator>>(unsigned short &);
   BL_Message& operator>>(int &);
   BL_Message& operator>>(unsigned int &);
   BL_Message& operator>>(long &);
   BL_Message& operator>>(unsigned long &);

   #ifdef BL_FP_IEEE_754
     BL_Message& operator>>(float &);
     BL_Message& operator>>(double &);
   #endif

   // Warning: these two functions dinamically allocate memory as
   // needed; NEVER allocate memory to the variables before using
   // these operators, because you would lose the opportunity to
   // release the memory previously reserved.

   BL_Message& operator>>(BL_Charp &);
   BL_Message& operator>>(BL_RawData &);

   // Insertion operators

   BL_Message& operator<<(char);
   BL_Message& operator<<(unsigned char);
   BL_Message& operator<<(short);
   BL_Message& operator<<(unsigned short);
   BL_Message& operator<<(int);
   BL_Message& operator<<(unsigned int);
   BL_Message& operator<<(long);
   BL_Message& operator<<(unsigned long);
   BL_Message& operator<<(BL_Int16);
   BL_Message& operator<<(BL_Uint16);
   BL_Message& operator<<(BL_Int32);
   BL_Message& operator<<(BL_Uint32);
   BL_Message& operator<<(BL_Int64);
   BL_Message& operator<<(BL_Uint64);
   BL_Message& operator<<(BL_Charp);
   BL_Message& operator<<(BL_RawData);

   #ifdef BL_FP_IEEE_754
     BL_Message& operator<<(float);
     BL_Message& operator<<(double);
   #endif

   // Miscellaneous methods

   unsigned long
   SeekItem (unsigned long);

   // Method to position the item pointer to the item corresponding
   // to the number indicated, or to the last item if the number is
   // too great.
   // Returns new item number.


   long
   SkipItem (long);

   // Method to skip onward/backward a certain number of items.
   // A positive value for iItemDisplacement move the item
   // pointer towards the end of the buffer; a negative value
   // moves the pointer towards the beginning of the buffer.
   // A null value for iItemDisplacement returns the current
   // item number.
   // Returns:
   //   number of really skipped items (iItemDisplacement!=0);
   //  or
   //   current item number (iItemDisplacement==0).


   int
   DeleteItem(unsigned long);

   // Method to delete the item corresponding to the number passed.
   // After this operation, the item is lost and can not be accessed anymore.
   // Item pointer, after operation, remains over the same item where
   // it was before; if this item was the one deleted, the pointer
   // is set to the next item (if exists).
   //
   // Returns:      BL_ITEM_DELETED    if item has been deleted
   //               BL_WRONG_ITEM      otherwise


   int
   GetItemType (void);

   // Method to retrieve the type of the current item.
   // Returns:
   //
   //            BL_NOT_VALID       Invalid item or item pointer
   //                               at the end of cpMsgBuf.
   //            BL_CHAR
   //            BL_UCHAR
   //            BL_I16
   //            BL_UI16
   //            BL_I32
   //            BL_UI32
   //            BL_I64
   //            BL_UI64
   //            BL_FLOAT
   //            BL_DOUBLE
   //            BL_CHARP
   //            BL_RAWDATA


   void
   SetType (BL_INT32);
   // Method used by the user to set the type tag


   BL_INT32
   GetType ();
   // Method to retrieve the type tag


   unsigned long
   GetDataLen ();
   // Method to retrieve total message buffer length


   int
   EOMsg ();


   void
   Clear ();


   int
   MsgSave (char * cpDestName);

   // Method to write a message to a file;
   // the file is created, if it does not exist, and automatically
   // truncated to zero length before writing if it already exists.
   // Return 0 if message was not saved succesfully, 1 otherwise.

   int
   MsgLoad (char * cpSourceName);

   // Method to read a message from a file
   // into the object attributes.
   // Return 0 if message was not loaded succesfully, 1 otherwise.

 }; // End of class BL_Message

 // Inline methods

 #include "bllibi.h"

#endif // End of conditional declarations

//******************************************************************
//************************** END OF FILE ***************************
//******************************************************************
